import numpy as np
from timeit import default_timer as timer
from numba import njit, prange, cuda
from numba.cuda.random import create_xoroshiro128p_states as curand_states
from numba.cuda.random import xoroshiro128p_uniform_float32 as curand_uniform

def monitor(guess_pi):
    guess_pi(2)
    ts = timer()
    pi, steps = guess_pi()
    perf = steps/(timer()-ts)
    return pi, steps, perf

def guess_pi_python(t=7):
    steps = 10 ** t
    pi = 0
    for i in range(steps):
        if np.random.rand()**2+np.random.rand()**2 < 1:
            pi += 1
    return pi*4/steps, steps

def guess_pi_numpy(t=8):
    steps = 10 ** t
    pi = np.sum((np.random.rand(steps)**2+np.random.rand(steps)**2)<1)
    return pi*4/steps, steps

@njit()
def guess_pi_jit(t=9):
    steps = 10 ** t
    pi = 0
    for i in range(steps):
        if np.random.rand()**2+np.random.rand()**2 < 1:
            pi += 1
    return pi*4/steps, steps

@njit(parallel = True)
def guess_pi_pjit(t=10):
    steps = 10 ** t
    pi = 0
    for i in prange(steps):
        if np.random.rand()**2+np.random.rand()**2 < 1:
            pi += 1
    return pi * 4 / steps, steps

@cuda.jit
def cuda_pi(rng_states, iterations, out):
    thread_id = cuda.grid(1)
    pi = 0
    for i in range(iterations):
        x = curand_uniform(rng_states, thread_id)
        y = curand_uniform(rng_states, thread_id)
        if x**2 + y**2 <= 1.0:
            pi += 1
    out[thread_id] = 4.0 * pi / iterations

def guess_pi_cuda(t=11):
    TpB = 2**10; BpG = 2**10
    iterations = np.math.ceil(10 ** t / TpB / BpG)
    rng_states = curand_states(TpB*BpG, seed=1)
    out = np.zeros(TpB*BpG, dtype=np.float32)
    cuda_pi[BpG, TpB](rng_states, iterations, out)
    return out.mean(), iterations*TpB*BpG