import numpy as np
import matplotlib.pyplot as plt
import matplotlib
from timeit import default_timer as timer
from numba import njit, prange, cuda
from view.plt_to_png import plt_to_png

matplotlib.use('AGG')

def draw_fractal(make_fractal):
    for i in range(10):
        make_fractal(100, 100)
    ts = timer()
    image = make_fractal()
    perf = (timer()-ts)
    plt.figure(figsize=(15, 8))
    plt.imshow(image)
    plt.xticks([], [])
    plt.yticks([], [])
    return perf, plt_to_png(plt)


def mandel_python(x, y, max_iters):
    c = complex(x, y)
    z = 0.0j
    for i in range(max_iters):
        z = z*z + c
        if (z.real*z.real + z.imag*z.imag) >= 4:
            return i
    return max_iters


def make_fractal_python(height = 2048, width = 3072, min_x=-2, max_x=1, min_y=-1, max_y=1, iters=20):
    image = np.zeros((height, width), dtype=np.uint8)
    pixel_size_x = (max_x - min_x) / width
    pixel_size_y = (max_y - min_y) / height
    for x in range(width):
        real = min_x + x * pixel_size_x
        for y in range(height):
            imag = min_y + y * pixel_size_y
            color = mandel_python(real, imag, iters)
            image[y, x] = color
    return image

def make_fractal_numpy(height = 2048, width = 3072, min_x=-2, max_x=1, min_y=-1, max_y=1, iters=20):
    image = iters*np.ones((height, width), dtype=np.uint8)
    c = np.dot((np.ones(height, dtype=complex)*np.linspace(max_y, min_y, height)).reshape(height,1),
            1.0j*np.ones((1, width), dtype=complex))
    c += np.dot(np.ones((height, 1)),
             np.linspace(min_x, max_x, width).reshape(1, width))
    z = np.zeros((height, width), dtype = complex)
    active = np.ones((height, width), dtype = bool)
    for i in range(iters):
        z[active] = z[active]*z[active] +c[active]
        mask = np.logical_and(active, z.real**2+z.imag**2 >= 4)
        image[mask] = i
        active[mask] = False
    image[active] = iters
    return image

@njit()
def mandel_jit(x, y, max_iters):
    c = complex(x, y)
    z = 0.0j
    for i in range(max_iters):
        z = z*z + c
        if (z.real*z.real + z.imag*z.imag) >= 4:
            return i
    return max_iters

@njit()
def make_fractal_jit(height = 2048, width = 3072, min_x=-2, max_x=1, min_y=-1, max_y=1, iters=20):
    image=iters*np.ones((height, width), dtype=np.uint8)
    pixel_size_x = (max_x - min_x) / width
    pixel_size_y = (max_y - min_y) / height
    for x in range(width):
        real = min_x + x * pixel_size_x
        for y in range(height):
            imag = min_y + y * pixel_size_y
            color = mandel_jit(real, imag, iters)
            image[y, x] = color
    return image

@njit(parallel = True)
def make_fractal_pjit(height = 2048, width = 3072, min_x=-2, max_x=1, min_y=-1, max_y=1, iters=20):
    image=iters*np.ones((height, width), dtype=np.uint8)
    pixel_size_x = (max_x - min_x) / width
    pixel_size_y = (max_y - min_y) / height
    for x in prange(width):
        real = min_x + x * pixel_size_x
        for y in prange(height):
            imag = min_y + y * pixel_size_y
            color = mandel_jit(real, imag, iters)
            image[y, x] = color
    return image

mandel_gpu = cuda.jit(device=True)(mandel_jit)

@cuda.jit
def mandel_kernel(min_x, max_x, min_y, max_y, image, iters):
    height = image.shape[0]
    width = image.shape[1]
    i, j = cuda.grid(2)
    if i<height and j<width:
        image[i, j] = mandel_gpu(min_x+j*(max_x - min_x) / width, min_y+i*(max_y - min_y) / height, iters)

def make_fractal_cuda(height = 2048, width = 3072, min_x=-2, max_x=1, min_y=-1, max_y=1, iters=20):
    image = np.zeros((height, width), dtype = np.uint8)
    TpB = (32, 32)
    BpG_x = np.math.ceil(height / TpB[0])
    BpG_y = np.math.ceil(width / TpB[1])
    BpG = (BpG_x, BpG_y)
    mandel_kernel[BpG, TpB](min_x, max_x, min_y, max_y, image, iters)
    return image