from numba import cuda

from .template import TEMPLATE, MENU
from model.darts import *
from model.fractal import *
from view.plt_to_png import plt_to_png

def make_html(html_dict):
    menu = ""
    for key, value in MENU.items():
        menu += """
        <li><a %s href="%s">%s</a></li>
        """ % ('class="active"' if key == html_dict['url'] else "", key, value)
    html_dict['menu'] = menu
    return TEMPLATE % html_dict


########## Darts #################

def dart_body_simple():
    pi, steps, perf = monitor(guess_pi_numpy)
    return (f"<p>"
            f"After having thrown {steps:.2e} darts at a speed of {perf:.2e} darts per second, "
            f"I guess that π should be close to {pi}"
            f"</p>"
            f"<img src='pictures/dart.jpg' width='800px'>")

DARTS_ALGO = [
    {'name': 'Python', 's': 'pyt',
     'function': guess_pi_python},
    {'name': 'Numpy', 's': 'np',
     'function': guess_pi_numpy},
    {'name': 'Numba', 's': 'jit',
     'function': guess_pi_jit},
    {'name': 'Parallel Numba', 's': 'pjit',
     'function': guess_pi_pjit},
]

if cuda.is_available():
    DARTS_ALGO.append({'name': 'CUDA', 's': 'cuda',
     'function': guess_pi_cuda})

def select_format(entry, s):
    selected = "selected" if entry['s'] == s else ""
    return f'<option value="{entry["s"]}" {selected}>{entry["name"]}</option>'

def options(entry_list, s):
    return "\n".join([select_format(entry, s) for entry in entry_list])

def dart_body(session=None, method=""):
    FORM = (f'<form action="/darts" method="post">'
            f'<p>'
            f'<label for="start">Select how to throw darts:</label></p><p>'
            f'<select name="method" onchange="this.form.submit()">'
            f'<option value=""> - Select throwing method - </option>'
            f'{options(DARTS_ALGO, method)}'
            f'</select></p>'
            f'</form>')

    if method == "":
        return FORM + f"<img src='data:image/png;base64,{show_perf(session)}'\>"
    for i, algo in enumerate(DARTS_ALGO):
        if algo['s'] == method:
            guess_pi = algo['function']
            name = algo['name']
            pi, steps, perf = monitor(guess_pi)
            session['dart'][i] = perf

    return (f"{FORM}"
            f"<p>"
            f"With the {name} approach, after having thrown {steps:.2e} darts at a speed of "
            f"{perf:.2e} darts per second, "
            f"I guess that π should be close to {pi}"
            f"</p>"
            f"<img src='data:image/png;base64,{show_perf(session)}'\>"
            f"<img src='pictures/dart.jpg' width='800px'>")



data_records = {
    "Pastaga":{
        "data": [238478.90198077293, 2631902.755337049, 18779687.946864776, 31136850.390557196, 238478.90198077293],
        "l": 4},
    "Precious":{
        "data": [459709.5581160391, 17459638.58449041, 46941956.45710598, 51663576.245240435, 459709.5581160391],
        "l": 4},
    "Atlantic":{
        "data": [936852.6195222065, 40082848.72666457, 88888894.27538304, 180835732.42605874, 936852.6195222065],
        "l": 4},
    "Hyperion":{
        "data": [912639.8539407903, 56902760.59430644, 114956670.6127556, 1523706208.5182524, 14060480465.576632],
        "l": 5}
                }

def show_perf(session, data=data_records):
    w = .15
    for i, (key, value) in enumerate(data.items()):
        c_data = np.array(value["data"])
        l = value["l"]
        plt.bar(np.arange(l)+i*w, c_data[:l], w, label = key)
    sdata = np.array(session['dart'])
    #data = array([perf_base, perf_numpy, perf_jit, perf_par_jit, perf_cuda])
    l = 5 if cuda.is_available() else 4
    plt.bar(np.arange(l)+len(data_records)*w, sdata[:l], w, label="This machine")
    names = ['Python', 'Numpy', 'Jit', 'Jit+//', 'CUDA']
    plt.xticks(np.arange(5) + len(data) * w / 2, names)
    plt.ylabel('Darts per second')
    plt.yscale('log')
    plt.legend()
    return plt_to_png(plt)

######## Mandel ########

def mandel_body_simple():
    perf, img = draw_fractal(make_fractal_jit)
    return (f"<p>Time to draw picture: {perf:.2e}s.</p>"
            f"<img src='data:image/png;base64,{img}' width='800px' \>")

MANDEL_ALGO = [
    {'name': 'Python', 's': 'pyt',
     'function': make_fractal_python},
    {'name': 'Numpy', 's': 'np',
     'function': make_fractal_numpy},
    {'name': 'Numba', 's': 'jit',
     'function': make_fractal_jit},
    {'name': 'Parallel Numba', 's': 'pjit',
     'function': make_fractal_pjit},
]

if cuda.is_available():
    MANDEL_ALGO.append({'name': 'CUDA', 's': 'cuda',
     'function': make_fractal_cuda})

def mandel_format(name, perf):
    return f"<li>With the {name} approach, the picture is computed in {perf:.2e} seconds.</li>"

def mandel_perf(session):
    items = '\n'.join([mandel_format(MANDEL_ALGO[i]['name'], session['mandel'][i])
                       for i in range(5) if session['mandel'][i] >0 ])
    return (f"<ul>"
            f"{items}"
            f"</ul>")

def mandel_body(session=None, method=""):
    FORM = (f'<form action="/mandel" method="post">'
            f'<p>'
            f'<label for="start">Select how to draw fractal:</label></p><p>'
            f'<select name="method" onchange="this.form.submit()">'
            f'<option value=""> - Select drawing method - </option>'
            f'{options(MANDEL_ALGO, method)}'
            f'</select></p>'
            f'</form>')

    if method == "":
        return FORM+mandel_perf(session)
    for i, algo in enumerate(MANDEL_ALGO):
        if algo['s'] == method:
            make_fractal = algo['function']
            perf, img = draw_fractal(make_fractal)
            session['mandel'][i] = perf

    return (f"{FORM}"
            f"{mandel_perf(session)}"
            f"<img src='data:image/png;base64,{img}' width='800px'>")

